# coding: utf-8
from __future__ import unicode_literals

from requests import Response

from .exceptions import BadQuery, HTTPError

try:
    from typing import TYPE_CHECKING
    if TYPE_CHECKING:
        from typing import Any, Dict, Optional, Text, Type, Union
        from .client import NuxeoClient
        from .models import Model
except ImportError:
    pass


class APIEndpoint(object):
    """
    Represents an API endpoint for Nuxeo, containing common patterns
    for CRUD operations.
    """

    def __init__(
        self,
        client,  # type: NuxeoClient
        endpoint=None,  # type: Optional[Text]
        headers=None,  # type: Optional[Dict[Text, Text]]
        cls=None,  # type: Optional[Type]
    ):
        # type: (...) -> None
        """
        Creates an instance of the APIEndpoint class.

        :param client: the authenticated REST client
        :param endpoint: the URL path to the resource endpoint
        :param headers: the extra HTTP headers
        :param cls: the Class to use when parsing results
        """
        self.client = client
        if endpoint:
            self.endpoint = '{}/{}'.format(client.api_path, endpoint)
        else:
            self.endpoint = client.api_path
        self.headers = headers or {}
        self._cls = cls

    def get(
        self,
        path=None,  # type: Optional[Text]
        cls=None,  # type: Optional[Type]
        raw=False,  # type: bool
        single=False,  # type: bool
        **kwargs  # type: Any
    ):
        # type: (...) -> Any
        """
        Gets the details for one or more resources.

        :param path: the endpoint (URL path) for the request
        :param cls: a class to use for parsing, if different
                    than the base resource
        :param raw: if True, directly return the content of
                    the response
        :param single: if True, do not parse as list
        :return: one or more instances of cls parsed from
                 the returned JSON
        """
        endpoint = self.endpoint

        if not cls:
            cls = self._cls

        if path:
            endpoint = '{}/{}'.format(endpoint, path)

        response = self.client.request('GET', endpoint, **kwargs)

        if isinstance(response, Response):
            if raw or response.status_code == 204:
                return response.content
            json = response.json()
        else:
            return response

        if cls is dict:
            return json

        if not single and isinstance(json, dict) and 'entries' in json:
            json = json['entries']

        if isinstance(json, list):
            return [cls.parse(resource, service=self) for resource in json]

        return cls.parse(json, service=self)

    def post(self, resource=None, path=None, raw=False, **kwargs):
        # type: (Optional[Any], Optional[Text], bool, Any) -> Any
        """
        Creates a new instance of the resource.

        :param resource: the data to post
        :param path: the endpoint (URL path) for the request
        :param raw: if False, parse the outgoing data to JSON
        :return: the created resource
        """
        if resource and not raw and not isinstance(resource, dict):
            if isinstance(resource, self._cls):
                resource = resource.as_dict()
            else:
                raise BadQuery(
                    'Data must be a Model object or a dictionary.')

        endpoint = self.endpoint

        if path:
            endpoint = '{}/{}'.format(endpoint, path)

        response = self.client.request(
            'POST', endpoint, data=resource, raw=raw, **kwargs)

        if isinstance(response, dict):
            return response
        return self._cls.parse(response.json(), service=self)

    def put(self, resource=None, path=None, **kwargs):
        # type: (Optional[Model], Optional[Text], Any) -> Any
        """
        Edits an existing resource.

        :param resource: the resource instance
        :param path: the endpoint (URL path) for the request
        :return: the modified resource
        """

        endpoint = '{}/{}'.format(self.endpoint, path or resource.uid)

        data = resource.as_dict() if resource else resource

        response = self.client.request('PUT', endpoint, data=data, **kwargs)

        if resource:
            return self._cls.parse(response.json(), service=self)

    def delete(self, resource_id):
        # type: (Text) -> None
        """
        Deletes an existing resource.

        :param resource_id: the resource ID to be deleted
        """

        endpoint = '{}/{}'.format(self.endpoint, resource_id)
        self.client.request('DELETE', endpoint)

    def exists(self, path):
        # type: (Text) -> bool
        """
        Checks if a resource exists.

        :param path: the endpoint (URL path) for the request
        :return: True if it exists, else False
        """
        endpoint = '{}/{}'.format(self.endpoint, path)

        try:
            self.client.request('GET', endpoint)
            return True
        except HTTPError as e:
            if e.status != 404:
                raise e
        return False



# coding: utf-8
from __future__ import unicode_literals

# Force parameters verification for all operations
CHECK_PARAMS = False

# Maximum file size before enforcing chunk upload
CHUNK_LIMIT = 10 * 1024 * 1024  # 10 Mio

# Chunk size to download files
CHUNK_SIZE = 8192  # 8 Kio

# API paths
DEFAULT_API_PATH = 'api/v1'
DEFAULT_URL = 'http://localhost:8080/nuxeo/'

"""
Default value for the:
    - 'X-Application-Name' HTTP header
    - 'applicationName' URL parameter
"""
DEFAULT_APP_NAME = 'Python client'

# Retries for each upload/chunk upload before abandoning
MAX_RETRY = 3

# Size of chunks for the upload
UPLOAD_CHUNK_SIZE = 256 * 1024  # 256 Kio



# coding: utf-8
from __future__ import unicode_literals

from .endpoint import APIEndpoint
from .exceptions import BadQuery
from .models import Directory, DirectoryEntry

try:
    from typing import TYPE_CHECKING
    if TYPE_CHECKING:
        from typing import Any, Dict, Text, Optional, Union
        from .client import NuxeoClient
except ImportError:
    pass


class API(APIEndpoint):
    """ Endpoint for directories. """
    def __init__(self, client, endpoint='directory', headers=None):
        # type: (NuxeoClient, Text, Optional[Dict[Text, Text]]) -> None
        super(API, self).__init__(
            client, endpoint=endpoint, cls=DirectoryEntry, headers=headers)

    def get(self, dir_name, dir_entry=None):
        # type: (Text, Optional[Text]) -> Union[Directory, DirectoryEntry]
        """
        Get the entries of a directory.

        If dir_entry is not None, return the corresponding entry.

        :param dir_name: the name of the directory
        :param dir_entry: the name of an entry
        :return: the directory entries
        """
        path = dir_name
        if dir_entry:
            path = '{}/{}'.format(path, dir_entry)

        entries = super(API, self).get(path=path)
        if dir_entry:
            return entries
        return Directory(directoryName=dir_name, entries=entries, service=self)

    def post(
        self,
        resource=None,  # type: Union[Directory, DirectoryEntry]
        dir_name=None,  # type: Optional[Text]
        **kwargs  # type: Any
    ):
        # type: (...) -> resource
        """
        Create a directory or an entry.

        :param resource: the directory/entry to create
        :param dir_name: the name of the directory
        :return: the created directory/entry
        """
        if dir_name:
            if not isinstance(resource, DirectoryEntry):
                raise BadQuery('The resource should be a directory entry.')
            resource.directoryName = dir_name
        return super(API, self).post(resource=resource, path=dir_name)

    create = post  # Alias for clarity

    def put(
        self,
        resource,  # type: Union[Directory, DirectoryEntry]
        dir_name,  # type: Text
    ):
        # type: (...) -> resource
        """
        Update an entry.

        :param resource: the entry to update
        :param dir_name: the name of the directory
        :return: the updated entry
        """
        path = '{}/{}'.format(dir_name, resource.uid)
        return super(API, self).put(resource, path=path)

    def delete(self, dir_name, dir_entry=None):
        # type: (Text, Optional[Text]) -> None
        """
        Delete a directory or an entry.

        :param dir_name: the name of the directory
        :param dir_entry: the name of the entry
        """
        path = dir_name
        if dir_entry:
            path = '{}/{}'.format(path, dir_entry)
        super(API, self).delete(path)

    def exists(self, dir_name, dir_entry=None):
        # type: (Text, Optional[Text]) -> bool
        """
        Check if a directory or an entry exists.

        :param dir_name: the name of the directory
        :param dir_entry: the name of the entry
        :return: True if it exists, else False
        """
        path = dir_name
        if dir_entry:
            path = '{}/{}'.format(path, dir_entry)
        return super(API, self).exists(path)
